/*
 
 Copyright (C) 2010,2011 Timo Lassmann <timolassmann@gmail.com>
 
 This file is part of delve.
 
 Delve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Delve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Delve.  If not, see <http://www.gnu.org/licenses/>.
 
 */


#include "interface.h"
#include "dbio.h"

struct db* init_db(struct db* db,struct parameters* param)
{
	int i = 0;
	int j;
	db = malloc(sizeof(struct db));
	
	db->bfa = 0;
	j = (int)strlen(param->infile[1]);
	db->dbname  = malloc(sizeof(char)*(j+1));
	for(i = 0; i < j ;i++){
		db->dbname[i] = param->infile[1][i];
	}
	db->dbname[j] = 0;
	return db;
}



struct db* main_read_db(struct db* db,struct parameters* param)
{
	int i,j;
	db = malloc(sizeof(struct db));
	db = db_read_seqio(db, param);
	
	j = (int)strlen(param->infile[2]);
	db->dbname  = malloc(sizeof(char)*(j+1));
	for(i = 0; i < j ;i++){
		db->dbname[i] = param->infile[2][i];
	}
	db->dbname[j] = 0;

	
	return db;
}

void main_write_db(struct db* db,struct parameters* param)
{
	db_write_seqio(db,param);
}

void free_db(struct db* db)
{
	int i; 
	for(i = 0; i < db->seq_info->num_seq;i++){
		free(db->seq_info->names[i]); 
	}
	free(db->dbname);
	free(db->seq_info->names);
	free(db->seq_info->sn_len);
	free(db->seq_info->len);
	free(db->seq_info->nuc_counts);
	free(db->seq_info);
	free(db->bfa);
	free(db);
}

void db_write_seqio(struct db*  db,struct parameters* param)
{
	FILE* file;
	char* outfile;
	int i,j;
	j = (int)strlen(param->infile[1]);
	outfile = malloc(sizeof(char)*(j+4));
	for(i = 0; i < j ;i++){
		outfile[i] = param->infile[1][i];
	}
	outfile[j] = '.';
	outfile[j+1] = 's';
	outfile[j+2] = 'i';
	outfile[j+3] = 0;
	
	if (!(file = fopen(outfile, "w" ))){
		fprintf(stderr,"Cannot open query file '%s'\n", outfile);
		exit(-1);
	}
	fprintf(file,"%u\n",db->seq_info->total_len);
	
	fprintf(file,"%f\n",db->seq_info->nuc_counts[0]);
	fprintf(file,"%f\n",db->seq_info->nuc_counts[1]);
	fprintf(file,"%f\n",db->seq_info->nuc_counts[2]);
	fprintf(file,"%f\n",db->seq_info->nuc_counts[3]);
	fprintf(file,"%f\n",db->seq_info->nuc_counts[4]);
	for(i = 0;i < db->seq_info->num_seq;i++){
		
		//free(db->seq_info->names);
		//free(db->seq_info->sn_len);
		//free(db->seq_info->len);
		fprintf(file,"%s\t%u\t%d\n",db->seq_info->names[i],db->seq_info->len[i],db->seq_info->sn_len[i]);
	}
	free(outfile);
	fclose(file);
}

struct db* db_read_seqio(struct db*  db,struct parameters* param)
{
	FILE* file;
	char* outfile;
	char name[MAX_LINE];
	int tmp_sn = 0;
	unsigned int tmp_len = 0;
	int storage_num_chr = 10; 
	int i,j;
	int read = 0;
	j = (int)strlen(param->infile[2]);
	outfile = malloc(sizeof(char)*(j+4));
	for(i = 0; i < j ;i++){
		outfile[i] = param->infile[2][i];
	}
	outfile[j] = '.';
	outfile[j+1] = 's';
	outfile[j+2] = 'i';
	outfile[j+3] = 0;
	
	
	db->seq_info = malloc(sizeof(struct db_seq_info));
	db->seq_info->names = malloc(sizeof(char*)* storage_num_chr);
	db->seq_info->sn_len = malloc(sizeof(int)* storage_num_chr);
	db->seq_info->len = malloc(sizeof(unsigned int)* storage_num_chr);
	db->seq_info->nuc_counts = malloc(sizeof(float)*5);
	if (!(file = fopen(outfile, "r" ))){
		fprintf(stderr,"Cannot open query file '%s'\n", outfile);
		exit(-1);
	}
	fscanf(file, "%u\n",&db->seq_info->total_len);
	fscanf(file, "%f\n",&db->seq_info->nuc_counts[0]);
	fscanf(file, "%f\n",&db->seq_info->nuc_counts[1]);
	fscanf(file, "%f\n",&db->seq_info->nuc_counts[2]);
	fscanf(file, "%f\n",&db->seq_info->nuc_counts[3]);
	fscanf(file, "%f\n",&db->seq_info->nuc_counts[4]);
	
	while(fscanf(file, "%s\t%u\t%d",name,&tmp_len,&tmp_sn)!= EOF){
		
		j = strlen(name);
		
		db->seq_info->names[read] = malloc(sizeof(unsigned char)*(j+1) );
		
		for(i =0; i <j;i++){
			db->seq_info->names[read][i] = name[i];
		}
		db->seq_info->names[read][j] = 0;
		
		//db->seq_info->names[read] = (unsigned char*)strdup(name);
		db->seq_info->sn_len[read] = tmp_sn;
		db->seq_info->len[read] = tmp_len;
		read++;
		if(read == storage_num_chr){
			storage_num_chr = storage_num_chr << 1;
			db->seq_info->names = realloc(db->seq_info->names,sizeof(char*)* storage_num_chr);
			db->seq_info->sn_len = realloc(db->seq_info->sn_len,sizeof(int)* storage_num_chr);
			db->seq_info->len = realloc(db->seq_info->len,sizeof(int)* storage_num_chr);
		}
	}
	fclose(file);
	free(outfile);
	db->seq_info->num_seq = read;
	if(!param->lightDB){
		db = read_bfa(db,param);
	}
	return db;

}

struct db* read_bfa(struct db*  db,struct parameters* param)
{
	char* bfa = 0;
	FILE* file;
	int i,j;
	
	
	j = (int)strlen(param->infile[2]);
	bfa = malloc(sizeof(char)*(j+5));
	for(i	= 0; i < j ;i++){
		bfa[i] = param->infile[2][i];
	}
	bfa[j] = '.';
	bfa[j+1] = 'b';
	bfa[j+2] = 'f';
	bfa[j+3] = 'a';
	bfa[j+4] = 0;
	if (!(file = fopen(bfa, "r" ))){
		//return 0;
		fprintf(stderr,"Cannot open file '%s'\n", bfa);
		exit(-1);
	}
	db->bfa = malloc(sizeof(unsigned int)* ((db->seq_info->total_len/8)+1));
	fread(db->bfa,sizeof(unsigned int), (db->seq_info->total_len/8) + 1, file);
	fclose(file);
	
	free(bfa);
	return db;
}

