/*
 
 Copyright (C) 2010,2011 Timo Lassmann <timolassmann@gmail.com>
 
 This file is part of delve.
 
 Delve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Delve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Delve.  If not, see <http://www.gnu.org/licenses/>.
 
 */

#include "interface.h"
#include "dbio.h"
#include "nuc_code.h"

unsigned int get_chr_offset(struct db_seq_info* si, char* chr)
{
	int i;
	for(i = 0; i < si->num_seq;i++){
		//fprintf(stderr,"%s	%s\n",chr,si->names[i]);
		if(!strcmp(chr,(char*)si->names[i])){
			return si->len[i];
		}
	}
	return 0xFFFFFFFFu;
}

unsigned char* get_chr_name(struct db_seq_info* si, unsigned int pos)
{
	int i,j;
	j = -1;
	for(i = 1; i< si->num_seq;i++){	
		if(pos < si->len[i]){
			j = i-1;
			break;
		}
	}
	if(j == -1){
		j = si->num_seq-1;
	}
	return si->names[j];
}

unsigned int  get_chr_pos(struct db_seq_info* si, unsigned int pos)
{
	int i,j;
	j = -1;
	for(i = 1; i< si->num_seq;i++){	
		if(pos < si->len[i]){
			j = i-1;
			break;
		}
	}
	if(j == -1){
		j = si->num_seq-1;
	}
	return pos - si->len[j];
}


struct db* add_seq_info(struct db* db, char* fasta)
{
	char tmp[MAX_LINE];
	unsigned int* buffer = 0;
	char* bfa;
	FILE* file = 0;
	double tmp_nuc_count[5]; 
	double sum = 0.0;
	int fd;
	int fwtest;
	int storage_num_chr = 10; 
	int in_name = 0;
	int was_control = 1;
	int string_pos = 0;
	int names_read = -1;
	int done = 0;
	int c,i,j,n,m;
	unsigned int code = 0;
	unsigned int code_count = 0;
	int bc = 0;
	
	j = (int)strlen(fasta);
	bfa = malloc(sizeof(char)*(j+5));
	for(i = 0; i < j ;i++){
		bfa[i] = fasta[i];
	}
	bfa[j] = '.';
	bfa[j+1] = 'b';
	bfa[j+2] = 'f';
	bfa[j+3] = 'a';
	bfa[j+4] = 0;
	//fprintf(stderr,"%s\n",outfile);
	buffer = malloc(sizeof(unsigned int)*BUFFER_SIZE);
	m = 0;
	db->seq_info = malloc(sizeof(struct db_seq_info));
	db->seq_info->names = malloc(sizeof(char*)* storage_num_chr);
	db->seq_info->sn_len = malloc(sizeof(int)* storage_num_chr);
	db->seq_info->len = malloc(sizeof(int)* storage_num_chr);
	
	
	db->seq_info->nuc_counts  = malloc(sizeof(float) * 5);
	
	db->seq_info->total_len = 0;
	
	for(i = 0; i < 5;i++){
		db->seq_info->nuc_counts[i] = 0.0f;
		tmp_nuc_count[i] = 0.0;
	}
	
	if (!(file = fopen(fasta, "r" ))){
		fprintf(stderr,"Cannot open fasta file '%s'\n", fasta);
		exit(-1);
	}
	
	if((fd = open(bfa,O_WRONLY)) == -1){
		creat(bfa,444);
	}
	if((fd = open(bfa,O_WRONLY)) == -1){
		fprintf(stderr,"cannot open %s\n",bfa);
		exit(-1);
	}
	while(! done){
		while((c = fgetc(file))){
			if(c == EOF){
				done = 1;
				break;
			}
			if(((c == '>') && was_control)){
				in_name = 1;
				names_read++;
			}else if(iscntrl((int)c)){
				if(in_name){
					tmp[string_pos] = 0;
					n = (int)strlen(tmp);
					if(names_read >= storage_num_chr){
						storage_num_chr = storage_num_chr << 1;
						db->seq_info->names = realloc(db->seq_info->names,sizeof(char*)* storage_num_chr);
						db->seq_info->sn_len = realloc(db->seq_info->sn_len,sizeof(int)* storage_num_chr);
						db->seq_info->len = realloc(db->seq_info->len,sizeof(int)* storage_num_chr);
					}
					db->seq_info->sn_len[names_read] = n;
					db->seq_info->len[names_read] = m;
					db->seq_info->names[names_read] = malloc(sizeof(char)* (n+1));
					for(i = 0; i < n;i++){
						db->seq_info->names[names_read][i] = tmp[i];
					}
					db->seq_info->names[names_read][n] = 0;
					string_pos = 0;
				}
				in_name = 0;
				was_control = 1;
			}else{
				if(in_name){
					tmp[string_pos] = c;
					string_pos++;
				}else{
					if(isalnum(c)){
						code |= nuc_code5[c] << code_count ;
						//fprintf(stderr,"%d\n",nuc_code5[c]);
						tmp_nuc_count[(int)nuc_code5[c]]+= 1.0f; 
						
						code_count+=4;
						m++;
					}
				}
				was_control = 0;
				
				if(code_count == 32){
					buffer[bc] = code;
					bc++;
					code = 0;
					if(bc == BUFFER_SIZE){
						fwtest = write(fd,buffer,sizeof(unsigned int)* BUFFER_SIZE);
						bc = 0;
					}
					code_count = 0;
				}
				
			}
		}
	}
	db->seq_info->total_len = m;
	if(code_count){
		code = code << ((8-code_count)*4);
		buffer[bc] = code;
		bc++;
	}
	
	if(bc){
		fwtest = write(fd,buffer,sizeof(unsigned int)* bc);
	}
	
	
	sum = 0.0;
	for(i = 0; i < 5;i++){
		sum += tmp_nuc_count[i];
	}
	for(i = 0; i < 5;i++){
		db->seq_info->nuc_counts[i] = tmp_nuc_count[i] / sum ;
	}
	
	close(fd);
	fclose(file);
	db->seq_info->num_seq = names_read+1;
	free(bfa);
	free(buffer);
	return db;
}


unsigned int get_sequence(struct db* db,unsigned char* seq,unsigned int start,unsigned int len,int strand)
{
	FILE* file;
	unsigned char* bin_arr = 0;
	char* bfa = 0;
	unsigned int i;
	int j;
	unsigned int l;
	int c = 0;
	unsigned char code = 0u;
	int wait = start & 0x1;

	char* dbfile = db->dbname;
	
	if(!db->bfa){
		j = (int)strlen(dbfile);
		bfa = malloc(sizeof(char)*(j+5));
		for(i = 0; i < j ;i++){
			bfa[i] = dbfile[i];
		}
		bfa[j] = '.';
		bfa[j+1] = 'b';
		bfa[j+2] = 'f';
		bfa[j+3] = 'a';
		bfa[j+4] = 0;
		
		bin_arr = malloc(sizeof(unsigned char)* len );
		
		if (!(file = fopen(bfa, "r" ))){
			fprintf(stderr,"Cannot open file '%s'\n", bfa);
			exit(-1);
		}
		
		fseek(file,start >> 1u, SEEK_SET);
		fread(bin_arr,sizeof(unsigned char), (len >> 1u) + 1u, file);
		fclose(file);
		
		l = (len >> 1u) + 1u;
		for(i = 0; i < l;i++){
			code = bin_arr[i];
			if(!wait){
				seq[c] = code  & 0xFu ;
				c++;
			}
			seq[c] = (code >> 4) & 0xFu ;
			c++;
			if(wait){
				wait--;
			}
		}
		if(c > len){
			c = len;
		}
		seq[c] = 0;	
		free(bin_arr);
	}else{
		bin_arr  = (unsigned char*) (db->bfa);
		bin_arr += (start >> 1u);
		l = (len >> 1u) + 1u;
		for(i = 0; i < l;i++){
			code = bin_arr[i];
			if(!wait){
				seq[c] = code  & 0xFu ;
				c++;
			}
			seq[c] = (code >> 4) & 0xFu ;
			c++;
			if(wait){
				wait--;
			}
		}
		if(c > len){
			c = len;
		}
		seq[c] = 0;	
		
	}
	if(strand){
		seq = reverse_complement2(seq, c);
	}
	free(bfa);
	return c;
}


unsigned char* reverse_complement2(unsigned char* p,int len)
{
	unsigned char* tmp = malloc(sizeof(unsigned char)*MAX_LINE);
	int i,c;
	c = 0;
	for(i =len-1; i >= 0;i--){
		tmp[c] = rev_nuc_code5[(int)p[i]];
		c++;
	}
	tmp[c] = 0;
	for(i= 0; i < len;i++){
		p[i] = tmp[i];
	}
	free(tmp);
	return p;
}

unsigned char* reverse_without_complement(unsigned char* p,int len)
{
	unsigned char* tmp = malloc(sizeof(unsigned char)*MAX_LINE);
	int i,c;
	c = 0;
	for(i =len-1; i >= 0;i--){
		tmp[c] = (int)p[i];
		c++;
	}
	tmp[c] = 0;
	for(i= 0; i < len;i++){
		p[i] = tmp[i];
	}
	free(tmp);
	return p;
}



