/*
 
 Copyright (C) 2010,2011 Timo Lassmann <timolassmann@gmail.com>
 
 This file is part of delve.
 
 Delve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Delve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with Delve.  If not, see <http://www.gnu.org/licenses/>.
 
 */

#include "interface.h"
#include "dbio.h"
#include "rand.h"
#include "delve.h"
#include "pHMM.h"

#include "nuc_code.h"
#include "read_sam.h"
#include "seed.h"
#include "simulate.h"
#include "fix.h"
#include "2ndorder.h"
#include "mdlhtml.h"

int main (int argc,char * argv[]) {
	struct parameters* param = 0;
	int nseed = -1 * (int) time(NULL);
	struct db* db = 0;
	int i;
	
	//fprintf(stderr,"%X\n",256);
	seed=(&nseed);
	
	init_nuc_code();

	param = interface(param,argc,argv);
	
	if(!strcmp(param->infile[0], "index")){ 
		db = init_db(db,param);
		db = add_seq_info(db,param->infile[1]);
		for(i = 0; i < db->seq_info->num_seq;i++){
			fprintf(stderr,"%d	%d	%u	%s\n",i,db->seq_info->sn_len[i],db->seq_info->len[i],db->seq_info->names[i]);
		}
		main_write_db(db,param);
		free_db(db);
	}
	
	if(!strcmp(param->infile[0],"seed")){
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		seed_controller_thread(param);
		//seed_controller(param);
		if(param->outfile){
			fclose(outfile);
		}
	}
	
	if(!strcmp(param->infile[0], "align")){
		param->num_query = 1000000;
		db =  main_read_db(db,param);
		
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		i = phmm_controller(db,param);
		if(param->outfile){
			fclose(outfile);
		}
		free_db(db);
	}	
	
	
	if(!strcmp(param->infile[0], "aln")){
		param->num_query = 1000000;
		db =  main_read_db(db,param);
		//fprintf(stderr,"
		param->local_weighting = 4;
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		i = phmm2_controller(db,param);
		if(param->outfile){
			fclose(outfile);
		}
		free_db(db);
	}	
	if(!strcmp(param->infile[0], "post2")){
		param->num_query = 200000;
		//fprintf(stderr,"%f	%f	%f	%f\n",param->random_model_prior, 1.0 - param->random_model_prior,prob2scaledprob(param->random_model_prior),prob2scaledprob(1.0f - param->random_model_prior));
		//exit(0);
		if(!param->outfile){
			fprintf(stderr,"You need to specify an output file using the -o option.\n");
			free_param(param);
			exit(-1);
		}
		db =  main_read_db(db,param);
		
		i = phmm2_posterior_controller(db,param);
		free_db(db);
	}
	
	
	if(!strcmp(param->infile[0], "post")){
		param->num_query = 200000;
		
		if(!param->outfile){
			fprintf(stderr,"You need to specify an output file using the -o option.\n");
			free_param(param);
			exit(-1);
		}
		db =  main_read_db(db,param);
		
		i = post_phmm_controller(db,param);
		free_db(db);
	}	
	
	
	if(!strcmp(param->infile[0], "realign")){
		param->num_query = 200000;
		db =  main_read_db(db,param);
		
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		i = realign_sam(db,param);
		if(param->outfile){
			fclose(outfile);
		}
		free_db(db);
	}	
	
	if(!strcmp(param->infile[0], "show2")){
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		make_mdl_html(param);
		if(param->outfile){
			fclose(outfile);
		}
	}
	
	
	if(!strcmp(param->infile[0], "show")){
		struct hmm* hmm = init_hmm(500, 500);
		param->hmm_file = param->infile[1];
		read_hmm_from_file(hmm,param);
		print_hmm_param(hmm); 
		free_hmm(hmm);
	}
	
	if(!strcmp(param->infile[0], "sim")){
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		simulate(param);
		if(param->outfile){
			fclose(outfile);
		}
	}
	
	if(!strcmp(param->infile[0], "bedsim")){
		param->num_query = 200000;
		
		db =  main_read_db(db,param);
		simulate_reads_from_bed(db,param);
		free_db(db);
	}	
	
	if(!strcmp(param->infile[0], "eval")){
		param->num_query = 200000;
		
		param->lightDB = 1;
		//db =  main_read_db(db,param);
		db = evaluate_sam_file(db,param);
		free_db(db);
	}	
	
	
	if(!strcmp(param->infile[0], "filter")){
		param->num_query = 200000;
		
		param->lightDB = 1;
		if(!param->outfile){
			outfile = stdout;
		}else{
			if ((outfile = fopen(param->outfile, "w")) == NULL){
				fprintf(stderr,"can't open output\n");
				exit(-1);
			}
		}
		
		
		
		
		//db =  main_read_db(db,param);
		db = fix_sam_file(db,param);
		if(param->outfile){
			fclose(outfile);
		}
		free_db(db);
	}	

	
	
	free_param(param);
	return 0;
}


















